from .base_api import BaseApi


class TravelApi(BaseApi):
    def __init__(self, options):
        super(TravelApi, self).__init__(options)
        self._default_params['datumType'] = 1

    def simple_hotel_search(self, **kwargs):
        return self._request(
            '/Travel/SimpleHotelSearch/20131024',
            kwargs,
            self._parse_hotels_result
        )

    def vacant_hotel_search(self, **kwargs):
        return self._request(
            '/Travel/VacantHotelSearch/20131024',
            kwargs,
            self._parse_hotels_result
        )

    def hotel_detail_search(self, **kwargs):
        return self._request(
            '/Travel/HotelDetailSearch/20131024',
            kwargs,
            self._parse_hotels_result
        )

    def get_area_class(self, **kwargs):
        return self._request(
            '/Travel/GetAreaClass/20131024',
            kwargs,
            self._parse_areas
        )

    def keyword_hotel_search(self, keyword, **kwargs):
        kwargs['keyword'] = keyword
        return self._request(
            '/Travel/KeywordHotelSearch/20131024',
            kwargs,
            self._parse_hotels_result
        )

    def get_hotel_chain_list(self, **kwargs):
        return self._request(
            '/Travel/GetHotelChainList/20131024',
            kwargs,
            self._parse_hotel_chain
        )

    def _parse_hotel_chain(self, result):
        chains = []
        for chainInfo in result['largeClasses'][0]['largeClass']:
            new_chains = [r['hotelChain'] for r in chainInfo['hotelChains']]
            chains.append({'largeClass': chainInfo['largeClassCode'], 'chains': new_chains})
        return chains

    def _parse_areas(self, result):
        final_res = result['areaClasses']['largeClasses'][0]['largeClass'][0]
        final_res['middle_classes'] = []
        middle_classes = result['areaClasses']['largeClasses'][0]['largeClass'][1]['middleClasses']
        for m in middle_classes:
            cl = m['middleClass'][0]
            cl['small_classes'] = []
            sub_classes = m['middleClass'][1]['smallClasses']
            for s in sub_classes:
                d = s['smallClass'][0]
                if len(s['smallClass']) > 1:
                    d['detail_classes'] = s['smallClass'][1]
                else:
                    d['detail_classes'] = []
                cl['small_classes'].append(d)
            final_res['middle_classes'].append(cl)
        return final_res

        r = result['areaClasses']['largeClasses'][0]['largeClass'][1]

    def _parse_hotels_result(self, result):
        return [self._parse_hotel(r) for r in result['hotels']]

    def _parse_hotel(self, hotel_info):
        hotel = hotel_info['hotel'][0]['hotelBasicInfo']
        room_info = [r['roomInfo'][0]['roomBasicInfo'] for r in hotel_info['hotel'] if 'roomInfo' in r]
        rating_info = [r['hotelRatingInfo'] for r in hotel_info['hotel'] if 'hotelRatingInfo' in r]
        hotel['room_info'] = room_info
        hotel['rating_info'] = rating_info
        return hotel



from setuptools import setup

setup(
    name='rakuten',
    version='0.1.5',
    author='Daniel Perez',
    author_email='daniel@claudetech.com',
    packages=['rakuten', 'rakuten.apis'],
    url='https://github.com/claudetech/python_rakuten',
    license='LICENSE',
    description='Rakuten API client.',
    long_description=open('README.md').read(),
    install_requires=[
         'requests >= 2.3.0'
    ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries'
    ],
)



import re
import json
import requests
from .api_exception import RakutenApiException

class BaseApi:
    def __init__(self, options):
        self._options = options
        self._base_url = self._options['api_endpoint']
        if self._base_url.endswith('/'):
            self._base_url = self._base_url[:-1]
        self._default_params = {
            'applicationId': self._options['app_id'],
            'format': 'json',
        }

    def _make_url(self, path):
        if not path.startswith('/'):
            path = path + '/'
        return self._base_url + path

    def _dict_to_camel_case(self, old_params):
        params = {}
        for (key, value) in old_params.items():
            params[self._to_camel_case(key)] = value
        return params

    def _to_camel_case(self, name):
        splitted = name.split('_')
        return splitted[0] + ''.join(map(lambda x: x.capitalize(), splitted[1:]))

    def _request(self, path, params, parser=None, method='get', payload=None):
        params = self._dict_to_camel_case(params)
        params.update(self._default_params)
        url = self._make_url(path)
        headers = {'content-type': 'application/json'}
        args = {'params': params, 'headers': headers}
        if payload is not None:
            args['data'] = json.dumps(payload)
        r = getattr(requests, method)(url, **args)
        if r.status_code == 200:
            result = r.json()
            if parser is not None:
                result = parser(result)
            return result
        else:
            raise RakutenApiException(r.status_code, r.text)



import http

from .apis import TravelApi

class RakutenClient:
    def __init__(self, app_id, **kwargs):
        self._options = {
           'api_endpoint': 'https://app.rakuten.co.jp/services/api',
           'app_id': app_id,
        }
        self._options.update(kwargs)

        self.travel = TravelApi(self._options)
